{*******************************************************}
{                                                       }
{       NetView Version 1.0                             }
{                                                       }
{       Copyright (c) 1999 Crits Vadim                  }
{                                                       }
{*******************************************************}

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ToolWin, Registry, Snmp, WinSock, ShellApi;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    File1: TMenuItem;
    View1: TMenuItem;
    Help1: TMenuItem;
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    TBtnRefresh: TToolButton;
    ImageList: TImageList;
    TBtnName: TToolButton;
    TBtnEndp: TToolButton;
    TBtnSave: TToolButton;
    miSave: TMenuItem;
    miSaveAs: TMenuItem;
    N1: TMenuItem;
    miExit: TMenuItem;
    miAbout: TMenuItem;
    miEndp: TMenuItem;
    miName: TMenuItem;
    N2: TMenuItem;
    miRefresh: TMenuItem;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ListView: TListView;
    PopupMenu: TPopupMenu;
    popOpen: TMenuItem;
    popAbout: TMenuItem;
    N3: TMenuItem;
    popExit: TMenuItem;
    miTrayIcon: TMenuItem;
    miTbr: TMenuItem;
    miSbr: TMenuItem;
    N4: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure TBtnRefreshClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure miSaveAsClick(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure miTbrClick(Sender: TObject);
    procedure miSbrClick(Sender: TObject);
    procedure miNameClick(Sender: TObject);
    procedure miEndpClick(Sender: TObject);
    procedure miAboutClick(Sender: TObject);
    procedure miTrayIconClick(Sender: TObject);
    procedure popOpenClick(Sender: TObject);
    procedure popExitClick(Sender: TObject);
    procedure ListViewColumnClick(Sender: TObject; Column: TListColumn);
  private
    { Private declarations }
    Env: TRegistry;
    IconNotifyData : TNotifyIconData;
    WindowPlacement: TWindowPlacement;
    TrayIcon, Action: Boolean;
    fName: string;
    procedure ApplicationMinimize(Sender : TObject);
    procedure WndProc(var Msg : TMessage); override;
    function GetPort(port: integer; proto: PChar): string;
    function GetHost(local: boolean; ipaddr: integer): string;
    procedure GetTcpUdpInfo;
    procedure MkFile(const name: string);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses About, Sort;

{$R *.DFM}

type
  PTcpInfo = ^TTcpInfo;
  TTcpInfo = packed record
    prev: PTcpInfo;
    next: PTcpInfo;
    state: UINT;
    localip: UINT;
    localport: UINT;
    remoteip: UINT;
    remoteport: UINT;
  end;

function SnmpExtensionInit(dwTimeZeroReference: DWORD; hPollForTrapEvent: PHandle;
                           supportedView: PAsnObjectIdentifier): BOOL; stdcall;
                           external 'inetmib1.dll' name 'SnmpExtensionInit';
function SnmpExtensionQuery(requestType: BYTE; variableBindings: PRFC1157VarBindList;
                            errorStatus, errorIndex: PAsnInteger): BOOL; stdcall;
                            external 'inetmib1.dll' name 'SnmpExtensionQuery';

const
  tcpidentifiers: array[0..9] of UINT = (1, 3, 6, 1, 2, 1, 6, 13, 1, 1);
  udpidentifiers: array[0..9] of UINT = (1, 3, 6, 1, 2, 1, 7, 5, 1, 1);
  TcpState: array[0..11]
  of string[13] = ('???',
                   'CLOSED',
                   'LISTENING',
                   'SYN_SENT',
                   'SEN_RECEIVED',
                   'ESTABLISHED',
                   'FIN_WAIT',
                   'FIN_WAIT2',
                   'CLOSE_WAIT',
                   'CLOSING',
                   'LAST_ACK',
                   'TIME_WAIT');

procedure TMainForm.FormCreate(Sender: TObject);
var
   WSAData: TWSAData;
begin
     if WSAStartup($0101, WSAData) <> 0 then
     begin
          MessageDlg('Could not initialize Winsock.', mtError, [mbOK], 0);
          Application.ShowMainForm := False;
          Application.Terminate;
     end;
     Application.OnMinimize := ApplicationMinimize;
     Env := TRegistry.Create;
     with Env do
     begin
          if not KeyExists('Software\TCPStat') then
          begin
               CreateKey('Software\TCPStat');
               OpenKey('Software\TCPStat', True);
          end
          else
          begin
               OpenKey('Software\TCPStat', True);
               if ReadInteger('WindowState') = 3 then
                  WindowState := wsMaximized
               else
               begin
                    if ValueExists('Top') then
                    begin
                         Top := ReadInteger('Top');
                         Left := ReadInteger('Left');
                         Height := ReadInteger('Height');
                         Width := ReadInteger('Width');
                    end;
               end;
               miTbr.Checked := ReadBool('ToolBar');
               miSbr.Checked := ReadBool('StatusBar');
               miName.Checked := ReadBool('Name');
               miEndp.Checked := ReadBool('Endp');
               if ReadBool('TrayIcon') then
               begin
                    Application.ShowMainForm := False;
                    miTrayIconClick(Self);
               end;
               ToolBar.Visible := miTbr.Checked;
               StatusBar.Visible := miSbr.Checked;
               TBtnName.Down := miName.Checked;
               TBtnName.ImageIndex := 3 - Ord(miName.Checked);
               TBtnEndp.Down := miEndp.Checked;
               TBtnEndp.ImageIndex := 5 - Ord(miEndp.Checked);
          end;
     end;
     with ListView do
     begin
          with Columns.Add do
          begin
               Caption:='Protocol';
               Width := 100;
          end;
          with Columns.Add do
          begin
               Caption:='Local Address';
               Width := -1;
          end;
          with Columns.Add do
          begin
               Caption:='Remote Address';
               Width := -1;
          end;
          with Columns.Add do
          begin
               Caption := 'State';
               Width := -1;
          end;
     end;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
     TBtnRefreshClick(Self);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     if TrayIcon then
     begin
          Action := caNone;
          Hide;
     end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
   ws: Integer;
begin
     WSACleanup;
     WindowPlacement.length := SizeOf(TWindowPlacement);
     GetWindowPlacement(Application.Handle, @WindowPlacement);
     ws := WindowPlacement.showCmd;
     if (ws <> SW_SHOWMINIMIZED) and (WindowState = wsMaximized) then
        ws := SW_SHOWMAXIMIZED;
     with Env do
     begin
          if WindowState = wsNormal then
          begin
               WriteInteger('Top', Top);
               WriteInteger('Left', Left);
               WriteInteger('Height', Height);
               WriteInteger('Width', Width);
          end;
          WriteBool('ToolBar', ToolBar.Visible);
          WriteBool('StatusBar', StatusBar.Visible);
          WriteBool('Name', TBtnName.Down);
          WriteBool('Endp', TBtnEndp.Down);
          WriteBool('TrayIcon', TrayIcon);
          WriteInteger('WindowState', ws);
          CloseKey;
     end;
end;

procedure TMainForm.TBtnRefreshClick(Sender: TObject);
begin
     StatusBar.SimpleText := 'Updating...';
     GetTcpUdpInfo;
     StatusBar.SimpleText := 'Ready';
end;

procedure TMainForm.miSaveClick(Sender: TObject);
begin
     if fName = '' then
        miSaveAsClick(Sender)
     else
        MkFile(fName);
end;

procedure TMainForm.miSaveAsClick(Sender: TObject);
var
   FileExt: string[4];
begin
     Application.ProcessMessages;
     with TSaveDialog.Create(Self) do
     try
        Filter := 'TCPStat Files (*.tcp)|*.tcp|';
        if Execute then
        begin
             fName := Filename;
             FileExt := ExtractFileExt(fName);
             if Copy(FileExt, 1, 1) = '.' then Delete(FileExt, 1, 1);
             if AnsiLowerCase(FileExt) = 'tcp' then
                FileName := fName
             else
             begin
                  Delete(fName, Pos('.', fName), Length(FileExt) + 1);
                  FileName := fName + '.' + 'tcp';
             end;
             if FileExists(FileName) then
             if MessageDlg(Format('%s already exists.' + #13#10 +
                'Do you want to replace it?', [FileName]),
                mtConfirmation, [mbYes, mbNo], 0) <> mrYes then Exit;
             fName := FileName;
             MkFile(fName);
        end;
     finally
        Free;
     end;
end;

procedure TMainForm.miExitClick(Sender: TObject);
begin
     Close;
end;

procedure TMainForm.miTbrClick(Sender: TObject);
begin
     ToolBar.Visible := not ToolBar.Visible;
     miTbr.Checked := not miTbr.Checked;
end;

procedure TMainForm.miSbrClick(Sender: TObject);
begin
     StatusBar.Visible := not StatusBar.Visible;
     miSbr.Checked := not miSbr.Checked;
end;

procedure TMainForm.miNameClick(Sender: TObject);
begin
     miName.Checked := not miName.Checked;
     TBtnName.Down := miName.Checked;
     TBtnName.ImageIndex := 3 - Ord(miName.Checked);
     TBtnRefreshClick(Self);
end;

procedure TMainForm.miEndpClick(Sender: TObject);
begin
     miEndp.Checked := not miEndp.Checked;
     TBtnEndp.Down := miEndp.Checked;
     TBtnEndp.ImageIndex := 5 - Ord(miEndp.Checked);
     TBtnRefreshClick(Self);
end;

procedure TMainForm.miTrayIconClick(Sender: TObject);
begin
     miTrayIcon.Checked := not miTrayIcon.Checked;
     TrayIcon := miTrayIcon.Checked;
     if TrayIcon then
     begin
          with IconNotifyData do
          begin
               hIcon := Application.Icon.Handle;
               uCallbackMessage := WM_USER + 1;
               cbSize := SizeOf(IconNotifyData);
               Wnd := Handle;
               uID := 100;
               uFlags := NIF_MESSAGE + NIF_ICON + NIF_TIP;
          end;
          StrPCopy(IconNotifyData.szTip, Application.Title);
          Shell_NotifyIcon(NIM_ADD, @IconNotifyData);
          ShowWindow(Application.Handle, SW_HIDE);
     end
     else
     begin
          Shell_NotifyIcon(NIM_DELETE, @IconNotifyData);
          ShowWindow(Application.Handle, SW_NORMAL);
     end;
end;

procedure TMainForm.miAboutClick(Sender: TObject);
begin
     with TAboutBox.Create(Self) do
     try
        ShowModal;
     finally
        Free;
     end;
end;

procedure TMainForm.popOpenClick(Sender: TObject);
begin
     WindowPlacement.length := SizeOf(TWindowPlacement);
     GetWindowPlacement(Application.Handle, @WindowPlacement);
     if WindowPlacement.showCmd = SW_SHOWMINIMIZED then
        Application.Restore;
     Show;
     ShowWindow(Application.Handle, SW_HIDE);
     Application.BringToFront;
end;

procedure TMainForm.popExitClick(Sender: TObject);
begin
     Shell_NotifyIcon(NIM_DELETE, @IconNotifyData);
     Application.ProcessMessages;
     Application.Terminate;
end;

procedure TMainForm.ListViewColumnClick(Sender: TObject;
  Column: TListColumn);
begin
     case Column.Index of
          0: if Action then
              begin
                   ListView.AlphaSort;
                   Action := False;
              end
              else
              begin
                   ListView.CustomSort(@ReverseSortItems, 0);
                   Action := True;
              end;
     1, 2, 3: if Action then
              begin
                   ListView.CustomSort(@SortSubItems, Column.Index - 1);
                   Action := False;
              end
              else
              begin
                   ListView.CustomSort(@ReverseSortSubitems, Column.Index - 1);
                   Action := True;
              end;
     end;
end;

procedure TMainForm.ApplicationMinimize(Sender : TObject);
begin
     if TrayIcon then ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TMainForm.WndProc(var Msg : TMessage);
var
   p: TPoint;
begin
     case Msg.Msg of
       WM_USER + 1:
       case Msg.lParam of
        WM_RBUTTONDOWN :begin
                             SetForegroundWindow(Handle);
                             GetCursorPos(p);
                             PopupMenu.Popup(p.x, p.y);
                             PostMessage(Handle, WM_NULL, 0, 0);
                        end;
        WM_LBUTTONDBLCLK: popOpenClick(PopupMenu);
       end;
     end;
     inherited;
end;

function TMainForm.GetPort(port: integer; proto: PChar): string;
var
   ServEnt: PServEnt;
   name: string[255];
begin
     ServEnt := GetServByPort(htons(port), proto);
     if (ServEnt <> nil) and (TBtnName.Down) then
	name := StrPas(ServEnt^.s_name)
     else
        name := IntToStr(port);
     Result := name;
end;

function TMainForm.GetHost(local: boolean; ipaddr: integer): string;
var
   HostEnt: PHostEnt;
   namelen: array[0..63] of Char;
   name: string[255];
begin
     if ipaddr = 0 then
     begin
          if (local) and (TBtnName.Down) then
          begin
               GetHostName(namelen, SizeOf(namelen));
               name := StrPas(namelen);
          end
          else
             name := Format('%d.%d.%d.%d', [0, 0, 0, 0]);
     end
     else
     begin
          if ipaddr = 16777343 then
          begin
               if TBtnName.Down then
                  name := 'localhost'
               else
               begin
                    HostEnt := GetHostByName('localhost');
                    name := inet_ntoa((PInAddr(HostEnt^.h_addr^))^);
               end;
          end
          else
          begin
               HostEnt := GetHostByAddr(@ipaddr, 4, PF_INET);
               if HostEnt <> nil  then
               begin
                    if TBtnName.Down then
                       name := StrPas(HostEnt^.h_name)
                    else
                       name := inet_ntoa((PInAddr(HostEnt^.h_addr^))^);
               end
               else
                  name := Format('%d.%d.%d.%d', [0, 0, 0, 0]);
          end;
     end;
     Result := name;
end;

procedure TMainForm.GetTcpUdpInfo;
var
   TcpInfoTable, UdpInfoTable: TTcpInfo;
   hTrapEvent: THandle;
   hIdentifier: TAsnObjectIdentifier;
   bindList: TRFC1157VarBindList;
   bindEntry: TRFC1157VarBind;
   errorStatus, errorIndex: TAsnInteger;
   currentEntry, newEntry: PTcpInfo;
   currentIndex: Integer;
   localaddr, remoteaddr: string;
   error: Boolean;
begin
     if not SnmpExtensionInit(GetCurrentTime, @hTrapEvent, @hIdentifier) then Exit;
     // TCP connections
     FillChar(bindList, SizeOf(bindList), 0);
     FillChar(bindEntry, SizeOf(bindEntry), 0);
     bindList.list := @bindEntry;
     bindList.len := 1;
     bindEntry.name.idLength := 10;
     bindEntry.name.ids := @tcpidentifiers;
     FillChar(TcpInfoTable, SizeOf(TcpInfoTable), 0);
     TcpInfoTable.prev := @TcpInfoTable;
     TcpInfoTable.next := @TcpInfoTable;
     currentIndex := 1;
     currentEntry := @TcpInfoTable;
     error := False;
     while not error do
     begin
          if not SnmpExtensionQuery(ASN_RFC1157_GETNEXTREQUEST,
                                    @bindList, @errorStatus, @errorIndex) then
          begin
               error := True;
               Continue;
          end;
          if bindEntry.name.idLength < 10 then Break;
          if currentIndex <> bindEntry.name.ids[9] then
          begin
               currentEntry := TcpInfoTable.next;
               currentIndex := bindEntry.name.ids[9];
          end;
          case bindEntry.name.ids[9] of
            1: begin
                    newEntry := PTcpInfo(AllocMem(sizeof(TTcpInfo)));
                    newEntry^.prev := currentEntry;
                    newEntry^.next := @TcpInfoTable;
                    currentEntry^.next := newEntry;
                    currentEntry := newEntry;
                    currentEntry^.state := bindEntry.value.number;
               end;
            2: begin
                    currentEntry^.localip := (PUINT(bindEntry.value.address.stream))^;
                    currentEntry := currentEntry^.next;
               end;
            3: begin
                    currentEntry^.localport := bindEntry.value.number;
                    currentEntry := currentEntry^.next;
               end;
            4: begin
                    currentEntry^.remoteip := (PUINT(bindEntry.value.address.stream))^;
                    currentEntry := currentEntry^.next;
               end;
            5: begin
                    currentEntry^.remoteport := bindEntry.value.number;
                    currentEntry := currentEntry^.next;
               end;
          end;
     end;
     with ListView.Items do
     begin
          BeginUpdate;
          Clear;
     end;
     currentEntry := TcpInfoTable.next;
     while currentEntry <> @TcpInfoTable do
     begin
          if not TBtnEndp.Down then if currentEntry^.state <> 5 then
          begin
               currentEntry := currentEntry^.next;
               Continue;
          end;
          localaddr := Format('%s: %s',
                              [GetHost(True, currentEntry^.localip),
                               GetPort(currentEntry^.localport, 'tcp')]);
          if currentEntry^.remoteip = 0 then
             remoteaddr := Format('%s: %s',
                                  [GetHost(False, currentEntry^.remoteip), '0'])
          else
             remoteaddr := Format('%s: %s',
                                  [GetHost(False, currentEntry^.remoteip),
                                   GetPort(currentEntry^.remoteport, 'tcp')]);
          with ListView.Items.Add do
          begin
               ImageIndex:=0;
               Caption := 'TCP';
               SubItems.Add(localaddr);
               SubItems.Add(remoteaddr);
               SubItems.Add(TcpState[currentEntry^.state]);
          end;
          currentEntry := currentEntry^.next;
     end;
     // UDP connections
     if TBtnEndp.Down then
     begin
          FillChar(bindList, SizeOf(bindList), 0);
          FillChar(bindEntry, SizeOf(bindEntry), 0);
          bindList.list := @bindEntry;
          bindList.len := 1;
          bindEntry.name.idLength := 10;
          bindEntry.name.ids := @udpidentifiers;
          FillChar(UdpInfoTable, SizeOf(UdpInfoTable), 0);
          UdpInfoTable.prev := @UdpInfoTable;
          UdpInfoTable.next := @UdpInfoTable;
          currentIndex := 1;
          currentEntry := @UdpInfoTable;
          error := False;
          while not error do
          begin
               if not SnmpExtensionQuery(ASN_RFC1157_GETNEXTREQUEST,
                                         @bindList, @errorStatus, @errorIndex) then
               begin
                    error := True;
                    Continue;
               end;
               if bindEntry.name.idLength < 10 then Break;
               if currentIndex <> bindEntry.name.ids[9] then
               begin
                    currentEntry := UdpInfoTable.next;
                    currentIndex := bindEntry.name.ids[9];
               end;
               case bindEntry.name.ids[9] of
                 1: begin
                         newEntry := PTcpInfo(AllocMem(sizeof(TTcpInfo)));
                         newEntry^.prev := currentEntry;
                         newEntry^.next := @UdpInfoTable;
                         currentEntry^.next := newEntry;
                         currentEntry := newEntry;
                         currentEntry^.localip := (PUINT(bindEntry.value.address.stream))^;
                    end;
                 2: begin
                         currentEntry^.localport := bindEntry.value.number;
                         currentEntry := currentEntry^.next;
                    end;
               end;
          end;
          currentEntry := UdpInfoTable.next;
          while currentEntry <> @UdpInfoTable do
          begin
               localaddr := Format('%s: %s',
                                   [GetHost(True, currentEntry^.localip),
                                    GetPort(currentEntry^.localport, 'udp')]);
               remoteaddr := '*.*.*.*: *';
               with ListView.Items.Add do
               begin
                    ImageIndex:=0;
                    Caption := 'UDP';
                    SubItems.Add(localaddr);
                    SubItems.Add(remoteaddr);
                    SubItems.Add('');
               end;
               currentEntry := currentEntry^.next;
          end;
     end;
     with ListView.Items do EndUpdate;
end;

procedure TMainForm.MkFile(const name: string);
var
   F: TextFile;
   i: Integer;
begin
     AssignFile(F, name);
     ReWrite(F);
     WriteLn(F, Format('%-10s%-39s%-41s%-13s%',
                       ['Protocol', 'Local Address',
                        'Remote Address', 'State']));
     for i := 0 to ListView.Items.Count - 1 do
         WriteLn(F, Format('%-10s%-39s%-41s%-13s',
                           [ListView.Items[i].Caption,
                            ListView.Items[i].SubItems.Strings[0],
                            ListView.Items[i].SubItems.Strings[1],
                            ListView.Items[i].SubItems.Strings[2]]));
     CloseFile(F);
end;

end.
